# -*- coding: utf-8 -*-
import os
import time
import Image
import wx
from grass.pygrass import modules

parallel = [True, False]
drivers = ["png", "cairo"]
bitmaps = ["png", "bmp", "ppm", "PPM"]
mmap_modes = ["FALSE", "TRUE"]
sizes = [1024, 4096]

############################################################################

def render_image(module, driver="png", pngfile="test.png",
                 size=4096, mapped="TRUE"):
    os.putenv("GRASS_RENDER_IMMEDIATE", "%s"%driver)
    os.putenv("GRASS_PNGFILE", "%s"%pngfile)
    os.putenv("GRASS_WIDTH", "%i"%size)
    os.putenv("GRASS_HEIGHT", "%i"%size)
    os.putenv("GRASS_PNG_MAPPED", "%s"%mapped)
    module.run()

############################################################################

def composite_images(files, bitmap, mode, size):
    start = time.time()
    if bitmap == "PPM":
        filename = "output"
        filename += ".ppm"
        modules.Module("g.pnmcomp", input=files, width=size, height=size,
                       output=filename)
        # Load the image as wx image for visualization
        img = wx.Image(filename, wx.BITMAP_TYPE_ANY)
        os.remove(filename)
        compositor="g.pnmcomp"
    else:
        images = []
        size = None
        for m in files:
            im = Image.open(m)
            images.append(im)
            size = im.size
        comp = Image.new('RGB', size)
        for im in images:
            comp.paste(im)
        wxImage = wx.EmptyImage(*comp.size)
        wxImage.SetData(comp.convert('RGB').tostring())
        compositor="PIL"

    return "%.3f  %s"%((time.time() - start),compositor)

############################################################################

def main():
    # Set the region
    modules.Module("g.region", rast="elevation", flags="p")

    for finish in parallel:
        if finish:
            print("*** Serial runs")
        else:
            print("*** Parallel runs")

        print("Run\tSize\tDriver\tBitmap\tmmap\trender\tcomposite")

        # Setup the modules
        rast   = modules.Module("d.rast", map="elevation", run_=False,
                                quiet=True, finish_=False)
        vectB = modules.Module("d.vect", map="streams", width=1, color="blue",
                                fcolor="aqua", type=["area","line"],
                                run_=False, quiet=True, finish_=finish)
        vectA = modules.Module("d.vect", map="roadsmajor", width=2,
                               run_=False, quiet=True, finish_=finish)

        count = 0
        for size in sizes:
            for driver in drivers:
                for bitmap in bitmaps:
                    for mode in mmap_modes:
                        # Skip mmap for non-bmp files
                        if mode == "TRUE" and bitmap != "bmp":
                            continue

                        start = time.time()
                        count += 1
                        files = []

                        rast_file = "rast.%s"%(bitmap.lower())
                        vectA_file="vectA.%s"%(bitmap.lower())
                        vectB_file="vectB.%s"%(bitmap.lower())

                        files.append(rast_file)
                        files.append(vectA_file)
                        files.append(vectB_file)

                        render_image(rast, driver=driver,
                                     pngfile=rast_file,
                                     size=size, mapped=mode)

                        render_image(vectA, driver=driver,
                                     pngfile=vectA_file,
                                     size=size, mapped=mode)

                        render_image(vectB, driver=driver,
                                     pngfile=vectB_file,
                                     size=size, mapped=mode)

                        # Wait for processes
                        rast.popen.wait()
                        vectA.popen.wait()
                        vectB.popen.wait()

                        # Composite the images
                        comptime = composite_images(files, bitmap, mode,
                                                    size)

                        for file in files:
                            os.remove(file)

                        elapsed = (time.time() - start)
                        print("%i\t%i\t%s\t%s\t%s\t%.3f\t%s"%(count, size,
                                                                driver, bitmap.lower(),
                                                                mode, elapsed,
                                                                comptime))

############################################################################

main()


